@@ -32,8 +32,12 @@
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
 #include <string.h>
-#include <musicbrainz/mb_c.h>
 
+#ifdef HAVE_LIBMUSICBRAINZ
+#include <musicbrainz/mb_c.h>
+#endif
+
+#include "totem-uri.h"
 #include "totem-interface.h"
 #include "totem-pl-parser.h"
 #include "debug.h"
@@ -95,6 +99,7 @@ struct TotemPlaylistPrivate
 /* Signals */
 enum {
 	CHANGED,
+	ITEM_ACTIVATED,
 	ACTIVE_NAME_CHANGED,
 	CURRENT_REMOVED,
 	REPEAT_TOGGLED,
@@ -392,37 +397,28 @@ drop_cb (GtkWidget     *widget,
 
 	for (p = file_list; p != NULL; p = p->next)
 	{
-		char *filename;
+		char *filename, *title;
 
 		if (p->data == NULL)
 			continue;
 
-		filename = gnome_vfs_get_local_path_from_uri (p->data);
-		if (filename == NULL)
-			filename = g_strdup (p->data);
+		filename = totem_create_full_path (p->data);
+		title = NULL;
 
-		if (filename != NULL &&
-				(g_file_test (filename, G_FILE_TEST_IS_REGULAR
-					| G_FILE_TEST_EXISTS)
-				 || strstr (filename, "://") != NULL))
+		if (info == 1)
 		{
-			char *title = NULL;
-
-			/* Super _NETSCAPE_URL trick */
-			if (info == 1)
-			{
-				g_free (p->data);
-				p = p->next;
-				if (p != NULL) {
-					if (g_str_has_prefix (p->data, "file:") != FALSE)
-						title = (char *) p->data + 5;
-					else
-						title = p->data;
-				}
+			g_free (p->data);
+			p = p->next;
+			if (p != NULL) {
+				if (g_str_has_prefix (p->data, "file:") != FALSE)
+					title = (char *)p->data + 5;
+				else
+					title = p->data;
 			}
-
-			totem_playlist_add_mrl (playlist, p->data, title);
 		}
+
+		totem_playlist_add_mrl (playlist, p->data, title);
+
 		g_free (filename);
 		g_free (p->data);
 	}
@@ -514,6 +510,7 @@ totem_playlist_set_reorderable (TotemPlaylist *playlist, gboolean set)
 		GtkTreeIter iter;
 		char *index;
 		GdkPixbuf *pixbuf;
+		GtkTreePath *path;
 
 		index = g_strdup_printf ("%d", i);
 		if (gtk_tree_model_get_iter_from_string
@@ -532,11 +529,19 @@ totem_playlist_set_reorderable (TotemPlaylist *playlist, gboolean set)
 			continue;
 
 		gdk_pixbuf_unref (pixbuf);
-		gtk_tree_path_free (playlist->_priv->current);
-		playlist->_priv->current = gtk_tree_path_new_from_indices (i, -1);
-		g_signal_emit (G_OBJECT (playlist),
-				totem_playlist_table_signals[CHANGED],
-				0, NULL);
+
+		/* Only emit the changed signal if we changed the ->current */
+		path = gtk_tree_path_new_from_indices (i, -1);
+		if (gtk_tree_path_compare (path, playlist->_priv->current) == 0)
+		{
+			gtk_tree_path_free (path);
+		} else {
+			gtk_tree_path_free (playlist->_priv->current);
+			playlist->_priv->current = path;
+			g_signal_emit (G_OBJECT (playlist),
+					totem_playlist_table_signals[CHANGED],
+					0, NULL);
+		}
 
 		break;
 	}
@@ -653,59 +658,28 @@ update_current_from_playlist (TotemPlaylist *playlist)
 static void
 totem_playlist_add_files (GtkWidget *widget, TotemPlaylist *playlist)
 {
-	GtkWidget *fs;
-	int response;
+	GSList *filenames, *l;
+	char *new_path;
 
-	fs = gtk_file_chooser_dialog_new (_("Select Movies or Playlists"),
-			totem_playlist_get_toplevel (playlist),
-			GTK_FILE_CHOOSER_ACTION_OPEN,
-			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-			GTK_STOCK_ADD, GTK_RESPONSE_ACCEPT,
-			NULL);
-	gtk_dialog_set_default_response (GTK_DIALOG (fs), GTK_RESPONSE_ACCEPT);
-	gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (fs), TRUE);
-	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (fs), FALSE);
+	filenames = totem_add_files (totem_playlist_get_toplevel (playlist),
+			playlist->_priv->path, &new_path);
+	if (filenames == NULL)
+		return;
 
-	if (playlist->_priv->path != NULL)
-	{
-		gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (fs),
-				playlist->_priv->path);
+	if (new_path != NULL) {
+		g_free (playlist->_priv->path);
+		playlist->_priv->path = new_path;
 	}
-	response = gtk_dialog_run (GTK_DIALOG (fs));
-	gtk_widget_hide (fs);
-	while (gtk_events_pending())
-		gtk_main_iteration();
 
-	if (response == GTK_RESPONSE_ACCEPT)
-	{
-		GSList *filenames, *l;
+	for (l = filenames; l != NULL; l = l->next) {
 		char *mrl;
 
-		filenames = gtk_file_chooser_get_uris (GTK_FILE_CHOOSER (fs));
-		if (filenames == NULL)
-		{
-			gtk_widget_destroy (fs);
-			return;
-		}
-
-		mrl = filenames->data;
-		if (mrl != NULL)
-		{
-			g_free (playlist->_priv->path);
-			playlist->_priv->path = g_path_get_dirname (mrl);
-		}
-
-		for (l = filenames; l != NULL; l = l->next)
-		{
-			mrl = l->data;
-			totem_playlist_add_mrl (playlist, mrl, NULL);
-			g_free (mrl);
-		}
-
-		g_slist_free (filenames);
+		mrl = l->data;
+		totem_playlist_add_mrl (playlist, mrl, NULL);
+		g_free (mrl);
 	}
 
-	gtk_widget_destroy (fs);
+	g_slist_free (filenames);
 }
 
 static void
@@ -1099,7 +1073,7 @@ treeview_row_changed (GtkTreeView *treeview, GtkTreePath *arg1,
 			(arg1, playlist->_priv->current) != FALSE)
 	{
 		g_signal_emit (G_OBJECT (playlist),
-				totem_playlist_table_signals[CHANGED], 0,
+				totem_playlist_table_signals[ITEM_ACTIVATED], 0,
 				NULL);
 		return;
 	}
@@ -1835,6 +1809,7 @@ totem_playlist_set_title (TotemPlaylist *playlist, const char *title)
 	return TRUE;
 }
 
+#ifdef HAVE_LIBMUSICBRAINZ
 #define CDDATALEN 256
 
 typedef struct _TotemCDData {
@@ -2074,10 +2049,12 @@ fail:
 
 	return NULL;
 }
+#endif
 
 void
 totem_playlist_set_cdindex (TotemPlaylist *playlist, const char *cdindex_id)
 {
+#ifdef HAVE_LIBMUSICBRAINZ
 	TotemCDData *data;
 
 	g_return_if_fail (cdindex_id != NULL);
@@ -2094,6 +2071,7 @@ totem_playlist_set_cdindex (TotemPlaylist *playlist, const char *cdindex_id)
 	data->list = playlist;
 
 	g_thread_create ((GThreadFunc) cd_metadata_thread, data, FALSE, NULL);
+#endif
 }
 
 gboolean
@@ -2101,6 +2079,7 @@ totem_playlist_set_playing (TotemPlaylist *playlist, gboolean state)
 {
 	GtkListStore *store;
 	GtkTreeIter iter;
+	GtkTreePath *path;
 
 	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), FALSE);
 
@@ -2122,6 +2101,16 @@ totem_playlist_set_playing (TotemPlaylist *playlist, gboolean state)
 		gtk_list_store_set (store, &iter,
 				PIX_COL, NULL,
 				-1);
+
+	if (state == FALSE)
+		return TRUE;
+
+	path = gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter);
+	gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (playlist->_priv->treeview),
+				      path, NULL,
+				      TRUE, 0.5, 0);
+	gtk_tree_path_free (path);
+	
 	return TRUE;
 }
 
@@ -2345,6 +2334,14 @@ totem_playlist_class_init (TotemPlaylistClass *klass)
 				NULL, NULL,
 				g_cclosure_marshal_VOID__VOID,
 				G_TYPE_NONE, 0);
+	totem_playlist_table_signals[ITEM_ACTIVATED] =
+		g_signal_new ("item-activated",
+				G_TYPE_FROM_CLASS (klass),
+				G_SIGNAL_RUN_LAST,
+				G_STRUCT_OFFSET (TotemPlaylistClass, item_activated),
+				NULL, NULL,
+				g_cclosure_marshal_VOID__VOID,
+				G_TYPE_NONE, 0);
 	totem_playlist_table_signals[ACTIVE_NAME_CHANGED] =
 		g_signal_new ("active-name-changed",
 				G_TYPE_FROM_CLASS (klass),
