@@ -35,10 +35,12 @@
 #include <gtk/gtk.h>
 
 struct TotemPropertiesViewPriv {
+	GtkWidget *label;
 	char *location;
 	GtkWidget *vbox;
 	BaconVideoWidgetProperties *props;
 	BaconVideoWidget *bvw;
+	guint timeout_id, try;
 };
 
 static GObjectClass *parent_class = NULL;
@@ -64,12 +66,58 @@ totem_properties_view_class_init (TotemPropertiesViewClass *class)
 static void
 on_got_metadata_event (BaconVideoWidget *bvw, TotemPropertiesView *props)
 {
+	GValue value = { 0, };
+	gboolean has_audio, has_video;
+	const char *label = NULL;
+
+	bacon_video_widget_get_metadata (BACON_VIDEO_WIDGET (bvw),
+			BVW_INFO_HAS_VIDEO, &value);
+	has_video = g_value_get_boolean (&value);
+	g_value_unset (&value);
+
+	bacon_video_widget_get_metadata (BACON_VIDEO_WIDGET (bvw),
+			BVW_INFO_HAS_AUDIO, &value);
+	has_audio = g_value_get_boolean (&value);
+	g_value_unset (&value);
+
+	if (has_audio == FALSE) {
+		if (has_video == FALSE) {
+			//FIXME this should be setting an error?
+			label = N_("Audio/Video");
+		} else {
+			label = N_("Video");
+		}
+	} else {
+		if (has_video == FALSE) {
+			label = N_("Audio");
+		} else {
+			label = N_("Audio/Video");
+		}
+	}
+
+	gtk_label_set_text (GTK_LABEL (props->priv->label), _(label));
+
 	bacon_video_widget_properties_update
-		(props->priv->props, props->priv->bvw, FALSE);
-	bacon_video_widget_close (props->priv->bvw);
-	g_free (props->priv->location);
-	props->priv->location = NULL;
+		(props->priv->props, props->priv->bvw);
 }
+
+static gboolean
+on_timeout_event (TotemPropertiesView *props)
+{
+	/* FIXME: hack for the GStreamer backend which signals metadata
+	 * in small chunks instead of all-at-once. */
+	if (props->priv->try++ >= 5) {
+		bacon_video_widget_close (props->priv->bvw);
+		g_free (props->priv->location);
+		props->priv->location = NULL;
+		props->priv->timeout_id = 0;
+
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 static void
 totem_properties_view_init (TotemPropertiesView *props)
 {
@@ -85,8 +133,6 @@ totem_properties_view_init (TotemPropertiesView *props)
 		/* Reference it, so that it's not floating */
 		g_object_ref (props->priv->bvw);
 
-		//FIXME
-
 		g_signal_connect (G_OBJECT (props->priv->bvw),
 				"got-metadata",
 				G_CALLBACK (on_got_metadata_event),
@@ -113,9 +159,14 @@ totem_properties_view_finalize (GObject *object)
 	if (props->priv != NULL)
 	{
 		g_object_unref (G_OBJECT (props->priv->bvw));
+		g_object_unref (G_OBJECT (props->priv->label));
 		props->priv->bvw = NULL;
 		g_free (props->priv->location);
 		props->priv->location = NULL;
+		if (props->priv->timeout_id != 0) {
+	                g_source_remove (props->priv->timeout_id);
+        	        props->priv->timeout_id = 0;
+		}
 		g_free (props->priv);
 	}
 	props->priv = NULL;
@@ -124,14 +175,16 @@ totem_properties_view_finalize (GObject *object)
 }
 
 GtkWidget *
-totem_properties_view_new (const char *location)
+totem_properties_view_new (const char *location, GtkWidget *label)
 {
-    TotemPropertiesView *self;
+	TotemPropertiesView *self;
 
-    self = g_object_new (TOTEM_TYPE_PROPERTIES_VIEW, NULL);
-    totem_properties_view_set_location (self, location);
+	self = g_object_new (TOTEM_TYPE_PROPERTIES_VIEW, NULL);
+	g_object_ref (label);
+	self->priv->label = label;
+	totem_properties_view_set_location (self, location);
 
-    return GTK_WIDGET (self);
+	return GTK_WIDGET (self);
 }
 
 void
@@ -146,8 +199,7 @@ totem_properties_view_set_location (TotemPropertiesView *props,
 		g_free(props->priv->location);
 		bacon_video_widget_close (props->priv->bvw);
 		props->priv->location = g_strdup (location);
-		bacon_video_widget_properties_update (props->priv->props,
-				props->priv->bvw, TRUE);
+		bacon_video_widget_properties_reset (props->priv->props);
 		if (bacon_video_widget_open (props->priv->bvw, location, &error) == FALSE) {
 			g_free (props->priv->location);
 			props->priv->location = NULL;
@@ -166,9 +218,11 @@ totem_properties_view_set_location (TotemPropertiesView *props,
 			g_error_free (error);
 			bacon_video_widget_close (props->priv->bvw);
 		}
+		props->priv->timeout_id =
+			g_timeout_add (200, (GSourceFunc) on_timeout_event,
+				       props);
 	} else {
-		bacon_video_widget_properties_update (props->priv->props,
-				props->priv->bvw, TRUE);
+		bacon_video_widget_properties_reset (props->priv->props);
 	}
 }
 
