@@ -23,15 +23,20 @@
 #include "config.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <string.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 
+#include "data/totem-mime-types.h"
 #include "totem-uri.h"
 #include "totem-private.h"
 
+static GtkFileFilter *filter_all;
+static GtkFileFilter *filter_supported;
+
 gboolean
 totem_playing_dvd (const char *uri)
 {
@@ -150,12 +155,9 @@ static const char *subtitle_ext[] = {
 char *
 totem_uri_get_subtitle_uri (const char *uri)
 {
-	char *suffix, *subtitle, *full;
+	char *suffix, *subtitle;
 	guint len, i;
-
-	if (g_str_has_prefix (uri, "file://") == FALSE) {
-		return NULL;
-	}
+	GnomeVFSURI *vfsuri;
 
 	if (strstr (uri, "#subtitle:") != NULL) {
 		return NULL;
@@ -166,27 +168,20 @@ totem_uri_get_subtitle_uri (const char *uri)
 		return NULL;
 	}
 
-	full = NULL;
 	subtitle = g_strdup (uri);
 	suffix = subtitle + len - 4;
 	for (i = 0; i < G_N_ELEMENTS(subtitle_ext) ; i++) {
-		char *fname;
-
 		memcpy (suffix + 1, subtitle_ext[i], 3);
-		fname = g_filename_from_uri (subtitle, NULL, NULL);
 
-		if (fname == NULL)
-			continue;
-
-		if (g_file_test (fname, G_FILE_TEST_IS_REGULAR) == FALSE) {
-			g_free (fname);
-			continue;
+		vfsuri = gnome_vfs_uri_new (subtitle);
+		if (vfsuri != NULL) {
+			if (gnome_vfs_uri_exists (vfsuri)) {
+				gnome_vfs_uri_unref (vfsuri);
+				return subtitle;
+			}
+		gnome_vfs_uri_unref (vfsuri);
 		}
-
-		g_free (fname);
-		return subtitle;
 	}
-
 	g_free (subtitle);
 	return NULL;
 }
@@ -219,3 +214,70 @@ totem_uri_escape_for_display (const char *uri)
 	return g_strdup (uri);
 }
 
+void
+totem_setup_file_filters (void)
+{
+	guint i;
+
+	filter_all = gtk_file_filter_new ();
+	gtk_file_filter_set_name (filter_all, _("All files"));
+	gtk_file_filter_add_pattern (filter_all, "*");
+
+	filter_supported = gtk_file_filter_new ();
+	gtk_file_filter_set_name (filter_supported,
+			_("Supported files"));
+	for (i = 0; i < G_N_ELEMENTS (mime_types); i++) {
+		gtk_file_filter_add_mime_type (filter_supported, mime_types[i]);
+	}
+}
+
+GSList *
+totem_add_files (GtkWindow *parent, const char *path, char **new_path)
+{
+	GtkWidget *fs;
+	int response;
+	GSList *filenames;
+	char *mrl;
+
+	fs = gtk_file_chooser_dialog_new (_("Select Movies or Playlists"),
+			parent,
+			GTK_FILE_CHOOSER_ACTION_OPEN,
+			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			GTK_STOCK_ADD, GTK_RESPONSE_ACCEPT,
+			NULL);
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (fs), filter_all);
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (fs), filter_supported);
+	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (fs), filter_supported);
+	gtk_dialog_set_default_response (GTK_DIALOG (fs), GTK_RESPONSE_ACCEPT);
+	gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (fs), TRUE);
+	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (fs), FALSE);
+
+	if (path != NULL) {
+		gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (fs),
+				path);
+	}
+
+	response = gtk_dialog_run (GTK_DIALOG (fs));
+	gtk_widget_hide (fs);
+	while (gtk_events_pending())
+		gtk_main_iteration();
+
+	if (response != GTK_RESPONSE_ACCEPT)
+		return NULL;
+
+	filenames = gtk_file_chooser_get_uris (GTK_FILE_CHOOSER (fs));
+	if (filenames == NULL) {
+		gtk_widget_destroy (fs);
+		return NULL;
+	}
+
+	mrl = filenames->data;
+	if (mrl != NULL && new_path != NULL) {
+		*new_path = g_path_get_dirname (mrl);
+	}
+
+	gtk_widget_destroy (fs);
+
+	return filenames;
+}
+
